/*
 * Copyright (c) 1992, 1993, 1994, 1995
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: bootbootp.c,v 1.24 95/04/12 02:16:00 leres Exp $ (LBL)
 */

#include <sys/param.h>
#include <sys/mount.h>
#include <sys/socket.h>

#include <net/if.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>

#include <arpa/inet.h>

#include <a.out.h>
#include <errno.h>
#ifdef TEST
#include <fcntl.h>
#include <unistd.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef LBL
#include <lbl/os-proto.h>
#endif

#include "bootbootp.h"
#include "tr.h"

#define BASEADDR 0x4000

/* Globals */
u_char	bcea[6] = BA;			/* broadcast ethernet address */
char	rootpath[FNAME_SIZE];		/* root mount path */
char	swappath[FNAME_SIZE];		/* swap mount path */
char	ifname[IFNAME_SIZE];		/* name of interface (e.g. "le0") */
n_long	rootip;				/* root ip address */
n_long	swapip;				/* swap ip address */
n_long	gateip;				/* swap ip address */
n_long	smask;				/* subnet mask */
n_long	nmask;				/* net mask */
n_long	mask;				/* subnet or net mask */
time_t	bot;				/* beginning of time in seconds */

int	nflag = 0;

/* Local forwards */
static	char *intoa __P((n_long));
int	main __P((int, char **));


#ifndef TEST
extern	struct promvec *promvec;
#endif

int
main(argc, argv)
	int argc;
	char **argv;
{
	register struct iodesc *d;
	struct iodesc iodesc;

	bot = getsecs();
	printf("%s loaded and running...\n", version);

	d = &iodesc;
	bzero(d, sizeof(*d));

	/* Setup house */
	ethernet_init(d);

	--argc;
	++argv;
	if (argc != 1) {
		printf("usage: target\n");
		exit(1);
	}

	ethernet_macaddr(d, sizeof(d->myea), d->myea);

	tr(d, argv[0]);

	exit(0);

}


/*
 * Convert Ethernet address to printable (loggible) representation.
 */
static char digits[] = "0123456789abcdef";
char *
ether_sprintf(ap)
        register u_char *ap;
{
	register i;
	static char etherbuf[18];
	register char *cp = etherbuf;

	for (i = 0; i < 6; i++) {
		*cp++ = digits[*ap >> 4];
		*cp++ = digits[*ap++ & 0xf];
		*cp++ = ':';
	}
	*--cp = 0;
	return (etherbuf);
}

/* Similar to inet_ntoa() */
static char *
intoa(addr)
	n_long addr;
{
	register char *cp;
	register u_int byte;
	register int n;
	static char buf[sizeof(".xxx.xxx.xxx.xxx")];

	cp = &buf[sizeof buf];
	*--cp = '\0';

	n = 4;
	do {
		byte = addr & 0xff;
		*--cp = byte % 10 + '0';
		byte /= 10;
		if (byte > 0) {
			*--cp = byte % 10 + '0';
			byte /= 10;
			if (byte > 0)
				*--cp = byte + '0';
		}
		*--cp = '.';
		addr >>= 8;
	} while (--n > 0);

	return (cp + 1);
}
