/*
 * Copyright (c) 1991, 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Rick Macklem at The University of Guelph.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)nfsdiskless.h	7.3+ (Berkeley) 2/6/92
 */

/*
 * Structure that must be initialized for a diskless nfs client. This
 * structure is used by nfs_mountroot() to set up the root and swap
 * vnodes plus do a partial ifconfig(8) and route(8) so that the
 * critical net interface can communicate with the server. This
 * currently only works for AF_INET protocols.
 *
 * The primary bootstrap is expected to fill in the appropriate fields
 * before starting vmunix.
 *
 * If the root or swap is not provided by the server, the corresponding
 * ip addresses are zero.
 *
 * If the root or swap is to be on a local device, the corresponding ip
 * address is set to INADDR_LOOPBACK (127.0.0.1) and the "name" string
 * specifies the device.
 *
 * NB: All fields are stored in net byte order to avoid hassles with
 * client/server byte ordering differences.
 */
struct nfs_diskless_v1 {
	long	version;			/* 1 */
	char	ifname[16];			/* if name, e.g. "en0" */
	n_long	addr;				/* my ip address */
	n_long	broadaddr;			/* my broadcast address */
	n_long	mask;				/* my net mask */
	n_long	mygateway;			/* Default gateway */

	u_char	swap_fh[32];			/* Swap file's file handle */
	n_long	swap_saddr;			/* Address of swap server */
	char	swap_hostnam[132];		/* Host name for mount pt */
	int	swap_nblks;			/* Size of server swap file */

	u_char	root_fh[32];			/* File handle of root dir */
	n_long	root_saddr;			/* Address of root server */
	char	root_hostnam[132];		/* Host name for mount pt */
	time_t	root_time;			/* Timestamp of root fs */

	char	my_hostnam[132];		/* Client host name */
};
