/*
 * Copyright (c) 1993, 1994, 1995
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: printf.c,v 1.5 95/04/12 18:29:25 leres Exp $ (LBL)
 */

/*
 * Version 3 PROM has no nice printf, so we roll our own.
 *
 * Note: EXTREMELY limited (handles only %c/%o/%d/%s/%m/%u/%x and literals).
 */

#include <sys/types.h>

#include <netinet/in_systm.h>

#include <errno.h>
#include <string.h>

#ifdef __STDC__
#include <stdarg.h>
#else
#define const
#include <varargs.h>
#endif

#ifdef LBL
#include <lbl/os-proto.h>
#endif

#include "bootbootp.h"
#include "printf.h"

int
vprintf(fmt, ap)
	register const char *fmt;
	va_list ap;
{
	register const char *cp;
	register char *s, *e;
	register int ret, c;
	register unsigned u;
	char numbuf[30];

	ret = 0;
	cp = fmt;
loop:
	while ((c = *cp) != '\0' && c != '%')
		cp++;
	ret += putstr(fmt, cp - fmt);
	if (c == '\0')
		goto done;
	++cp;		/* point to control char */
	if (*cp == 'l' && (cp[1] == 'd' || cp[1] == 'u' || cp[1] == 'x'))
		++cp;	/* skip "long" specifier */
	fmt = cp + 1;	/* (and past it, assuming successful interpretation) */
	switch (*cp) {	/* figure out what to do */

	case '\0':	/* oops, trailing % */
		goto done;

	case 's':	/* %s = string */
		s = va_arg(ap, char *);
		for (e = s; *e; e++)
			continue;
		ret += putstr(s, e - s);
		break;

	case 'c':	/* %c = character */
		c = va_arg(ap, int);
		putch(c);
		++ret;
		break;

	case 'm':	/* errno text */
		cp = strerror(errno);
		ret += putstr(cp, strlen(cp));
		break;

	case 'o':
		u = va_arg(ap, unsigned);
		s = &numbuf[sizeof numbuf];
		do {
			*--s = (u % 8) + '0';
			u /= 8;
		} while (u != 0);
		ret += putstr(s, &numbuf[sizeof numbuf] - s);
		break;

	case 'd':	/* signed int */
		u = va_arg(ap, unsigned);
		if ((int)u < 0) {
			u = -u;
			c = 1;
		} else
			c = 0;
		s = &numbuf[sizeof numbuf];
		do {
			*--s = (u % 10) + '0';
			u /= 10;
		} while (u != 0);
		if (c)
			*--s = '-';
		ret += putstr(s, &numbuf[sizeof numbuf] - s);
		break;

	case 'u':	/* unsigned int */
		u = va_arg(ap, unsigned);
		s = &numbuf[sizeof numbuf];
		do {
			*--s = (u % 10) + '0';
			u /= 10;
		} while (u != 0);
		ret += putstr(s, &numbuf[sizeof numbuf] - s);
		break;

	case 'x':	/* %x = number, in hex */
		u = va_arg(ap, unsigned);
		s = &numbuf[sizeof numbuf];
		do {
			*--s = "0123456789abcdef"[u & 15];
			u >>= 4;
		} while (u != 0);
		ret += putstr(s, &numbuf[sizeof numbuf] - s);
		break;

	default:
		/* %% or not understood */
		fmt--;	/* include the mystery char in later output */
		goto loop;
	}
	cp = fmt;
	goto loop;
done:
	return (ret);
}

int
#ifdef __STDC__
printf(const char *fmt, ...)
#else
printf(fmt, va_alist)
	char *fmt;
	va_dcl
#endif
{
	register int ret;
	va_list ap;

#ifdef __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	ret = vprintf(fmt, ap);
	va_end(ap);
	return (ret);
}
