/*
 * Copyright (c) 1995
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: prom_bsd.c,v 1.11 94/06/01 20:59:06 leres Locked $ (LBL)
 */

#include <sys/param.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#ifndef	SOCK_STREAM
#include <sys/socket.h>
#endif
#include <sys/time.h>

#include <netinet/in_systm.h>

#include <net/bpf.h>
#include <net/if.h>
#include <net/if_types.h>
#include <net/if_dl.h>

#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define printf Printf
#include <stdio.h>
#undef printf

#ifdef LBL
#include <lbl/os-proto.h>
#endif

#include "bootbootp.h"

/* Code stolen from myetheraddr */

#define NUMINTERFACES 16

/* Cast a struct sockaddr to a structaddr_dl */
#define SATOSDL(sa) ((struct sockaddr_dl *)(sa))

void
ethernet_macaddr(d, len, ea)
	register struct iodesc *d;
	int len;
	u_char *ea;
{
	register struct ifreq *ifrp, *ifend;
	register int fd;
	register int n;
	struct ifreq ibuf[NUMINTERFACES];
	struct ifconf ifc;

	if (len != 6)
		panic("ethernet_macaddr: bad buffer size %d", len);

	/* Fetch the interface configuration */
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd < 0)
		panic("ethernet_macaddr: socket: %s", strerror(errno));
	ifc.ifc_len = sizeof(ibuf);
	ifc.ifc_buf = (caddr_t)ibuf;
	if (ioctl(fd, SIOCGIFCONF, (char *)&ifc) < 0 ||
	    ifc.ifc_len < sizeof(struct ifreq))
		panic("ethernet_macaddr: SIOCGIFCONF: %s", strerror(errno));
	(void)close(fd);

	/* Spin through interface configuration list, setup internal list */
	ifrp = ibuf;
	ifend = (struct ifreq *)((char *)ibuf + ifc.ifc_len);
	while (ifrp < ifend) {
		if (ifrp->ifr_addr.sa_family == AF_LINK &&
		SATOSDL(&ifrp->ifr_addr)->sdl_type == IFT_ETHER) {
			bcopy((char *)LLADDR(SATOSDL(&ifrp->ifr_addr)), ea, 6);
			return;
		}

		/* Bump interface config pointer */
		n = ifrp->ifr_addr.sa_len + sizeof(ifrp->ifr_name);
		if (n < sizeof(*ifrp))
			n = sizeof(*ifrp);
		ifrp = (struct ifreq *)((char *)ifrp + n);
	}

	panic("ethernet_macaddr: can't find ethernet addr");
}
