/*
 * Copyright (c) 1992, 1993, 1994, 1995
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: prom_sparc.c,v 1.19 95/05/11 23:48:45 leres Exp $ (LBL)
 */

#include <sys/types.h>

#ifdef LBL
#include <lbl/os-proto.h>
#endif

#include <netinet/in_systm.h>

#include <machine/bsd_openprom.h>

#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#ifdef __STDC__
#include <stdarg.h>
#else
#define const
#include <varargs.h>
#endif

#include "bootbootp.h"
#include "printf.h"

/* XXX this is from /sys/sparc/sparc/clockreg.h which we can't include */
struct idprom {
	u_char	id_format;		/* format identifier (= 1) */
	u_char	id_machine;		/* machine type (see cpu.h) */
	u_char	id_ether[6];		/* ethernet address */
	long	id_xxx0;		/* ??? */
	u_char	id_hostid[3];		/* ``host id'' bytes */
	u_char	id_checksum;		/* xor of everything else */
	char	id_xxx1[16];		/* ??? */
};

/* Forwards */
void	prommain __P((void));
static	void rawputs __P((const char *, int));

struct	promvec *promvec;
int	promvers = -1;
int	errno;

static char bootargs[256];

void
prommain()
{
	register char *ip, **ap;
	register int argc;
	char *argv[64];
	char buf[256];
	register char *cp;
	register char c;
	extern int main __P((int, char **));

	argc = 0;
	ap = argv;
	cp = buf;

	*ap++ = "bootbootp";
	++argc;

	(void)strcpy(ifname, "??0");
	promvers = promvec->pv_romvec_vers;
	switch (promvers) {

	case 0:
		/* v0 prom */
		ip = (*promvec->pv_v0bootargs)->ba_args;

		/* Pick up ifname (v0 can only hack unit zero) */
		cp = ifname;
		if (ip[0] != '\0' && ip[1] != '\0' && ip[2] == '('){
			*cp++ = *ip++;
			*cp++ = *ip++;
			*cp++ = '0';
			++ip;
		}

		/* Step over "le(...)" part. */
		for (cp = ip; *cp != '\0'; ++cp)
			if (*cp == ')') {
				ip = cp + 1;
				break;
			}
		/* Stupid v0 prom is missing an initial newline */
		printf("\n");

		/* XXX Only need for SunOS (v0 is always a sun4c) */
		_fixsegmap();
		break;

	case 2:
	case 3:
		/* v2/v3 prom */
		cp = *promvec->pv_v2bootargs.v2_bootpath;
		/* Find last '/' */
		ip = cp;
		while (*cp != '\0')
			if (*cp++ == '/')
				ip = cp;

		if (ip != NULL && *ip != '\0') {
			/* Copy interface name */
			cp = ifname;
			while (*ip >= 'a' && *ip <= 'z')
				*cp++ = *ip++;

			/*
			 * The number after the '@' is the sbus slot
			 * number. Unfortunately, there's no easy way
			 * for us to map sbus slot numbers to kernel
			 * unit numbers so we just assume unit 0.
			 */
			*cp++ = '0';
			*cp = '\0';
		}
		ip = *promvec->pv_v2bootargs.v2_bootargs;
		break;

	default:
		panic("prommain: Can't handle v%d prom", promvers);
		break;
	}
#ifdef notdef
printf("v%d ifname \"%s\", args \"%s\"\n", promvers, ifname, ip);
#endif

	/* Copy the boot args to a static buffer and split into argv/argc */
	(void)strncpy(bootargs, ip, sizeof(bootargs) - 1);
	bootargs[sizeof(bootargs) - 1] = '\0';
	ip = bootargs;
	for (;;) {
		while ((c = *ip) == ' ' || c == '\t')
			++ip;
		if (c == '\0')
			break;
		*ap++ = ip;
		++argc;
		while ((c = *ip) != ' ' && c != '\t' && c != '\0')
			++ip;
		*ip = '\0';
		if (c != '\0')
			++ip;
	}
	*ap = NULL;

	(void)main(argc, argv);
	promvec->pv_halt();
}

void
ethernet_init(d)
	register struct iodesc *d;
{
	register char *name, *cp;
	char buf[256];

	switch (promvers) {

	case 0:
		name = buf;
		strncpy(name, (*promvec->pv_v0bootargs)->ba_args, sizeof(buf));
		name[sizeof(buf) - 1] = '\0';
		/* Truncate after "le(...)" part */
		for (cp = name; *cp != '\0'; ++cp)
			if (*cp == ')') {
				*++cp = '\0';
				break;
			}
		d->fd = promvec->pv_v0devops.v0_open(name);
		break;

	case 2:
	case 3:
	default:
		name = *promvec->pv_v2bootargs.v2_bootpath;
		d->fd = promvec->pv_v2devops.v2_open(name);
		break;
	}
	if (d->fd == 0)
		panic("ethernet_init: open %s failed", name);
}

void
ethernet_shutdown(d)
	register struct iodesc *d;
{

	switch (promvers) {

	case 0:
		(void)promvec->pv_v0devops.v0_close(d->fd);
		break;

	case 2:
	case 3:
	default:
		promvec->pv_v2devops.v2_close(d->fd);
		break;
	}
	d->fd = 0;
}

/* Returns the actual number of characters written */
int
ethernet_put(d, pkt, len)
	register struct iodesc *d;
	register void *pkt;
	register int len;
{
	register int n;

	switch (promvers) {

	case 0:
		n = promvec->pv_v0devops.v0_wnet(d->fd, len, pkt);
		break;

	case 2:
	case 3:
	default:
		n = promvec->pv_v2devops.v2_write(d->fd, pkt, len);
		break;
	}
	return (n);
}

/* Returns character count or -1 if none. No other error is possible */
int
ethernet_poll(d, pkt, len)
	register struct iodesc *d;
	register void *pkt;
	register int len;
{
	register int cc;

	switch (promvers) {

	case 0:
		cc = promvec->pv_v0devops.v0_rnet(d->fd, len, pkt);
		break;

	case 2:
	case 3:
	default:
		cc = promvec->pv_v2devops.v2_read(d->fd, pkt, len);
		break;
	}
	return (cc);
}

/* Returns character count or -1 on timeout. No other error is possible */
int
ethernet_get(d, pkt, len, timeout)
	register struct iodesc *d;
	register void *pkt;
	register int len;
	register time_t timeout;
{
	register time_t t;
	register int cc;

	t = getsecs();
	/* Spin, baby, spin! */
	do {
		cc = ethernet_poll(d, pkt, len);
	} while (cc < 0 && getsecs() - t < timeout);
	return (cc);
}

void
ethernet_macaddr(d, len, p)
	register struct iodesc *d;
	register int len;
	register u_char *p;
{
	register int rootnode;
	struct idprom idprom;

	if (len != 6)
		panic("ethernet_macaddr: bad buffer size %d", len);
	rootnode = promvec->pv_nodeops->no_nextnode(0);
	if (promvec->pv_nodeops->no_proplen(rootnode,
	    "idprom") > sizeof(idprom))
		panic("ethernet_macaddr: idprom too big");
	promvec->pv_nodeops->no_getprop(rootnode, "idprom",
	    (char *)&idprom);
	bcopy(idprom.id_ether, p, 6);
}

u_long
gettenths()
{

	return (*promvec->pv_ticks / 100);
}

time_t
getsecs()
{

	return ((time_t)(*promvec->pv_ticks / 1000));
}

/* Print a formatted message (with a trailing newline) and die */
__dead void
panic(char const *fmt, ...)
{
	register char *cp;
	va_list ap;

	va_start(ap, fmt);
	vprintf(fmt, ap);

	/* Output trailing newline if missing in format */
	for (cp = (char *)fmt; *cp != '\0'; ++cp)
		continue;
	if (--cp >= fmt && *cp != '\n')
		printf("\n");

	va_end(ap);
	promvec->pv_halt();
}

char *
strerror(num)
	register int num;
{
#define UPREFIX "Unknown error: "
	static char ebuf[40] = UPREFIX;		/* 64-bit number + slop */
	register unsigned int errnum;
	register char *p, *t;
	char tmp[40];

	errnum = num;				/* convert to unsigned */
	switch (errnum) {

	case EPERM:
		return ("Operation not permitted");

	case ENOENT:
		return ("No such file or directory");

	case EACCES:
		return ("Permission denied");

	case ESTALE:
		return ("Stale NFS file handle");
	}

	/* Do this by hand, so we don't include stdio(3). */
	t = tmp;
	do {
		*t++ = "0123456789"[errnum % 10];
	} while ((errnum /= 10) != 0);
	for (p = ebuf + sizeof(UPREFIX) - 1;;) {
		*p++ = *--t;
		if (t <= tmp)
			break;
	}
	return (ebuf);
}

int
putch(c)
	int c;
{
	char ch;

	switch (promvers) {

	case 0:
		(*promvec->pv_putchar)(c);
		break;

	case 2:
	case 3:
	default:
		if (c == '\n')
			promvec->pv_v2devops.v2_write(
			    *promvec->pv_v2bootargs.v2_fd1, "\r\n", 2);
		else {
			ch = c;
			promvec->pv_v2devops.v2_write(
			    *promvec->pv_v2bootargs.v2_fd1, &ch, 1);
		    }
		break;
	}
	return (c);
}

static void
rawputs(s, len)
	const char *s;
	int len;
{
	register int n;

	switch (promvers) {

	case 0:
		for (n = len; --n >= 0;)
			promvec->pv_putchar(*s++);
		break;

	case 2:
	case 3:
	default:
		promvec->pv_v2devops.v2_write(
		    *promvec->pv_v2bootargs.v2_fd1, (char *)s, len);
		break;
	}
}

int
putstr(s, len)
	const char *s;
	int len;
{
	register const char *cp;
	register int n, ret;

	cp = s;
	ret = len;
	n = len;
	while (n-- > 0)
		if (*cp++ == '\n') {
			n = len - n;
			rawputs(s, n - 1);
			rawputs("\r\n", 2);
			len -= n;
			s += n;
			n = len;
		}
	/* Put out rest of string */
	if (len > 0)
		rawputs(s, len);
	return (ret);
}
