/*
 * Copyright (c) 1995
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: prom_sunos.c,v 1.1 95/04/12 02:14:29 leres Exp $ (LBL)
 */

#include <sys/types.h>
#ifndef	SOCK_STREAM
#include <sys/socket.h>
#endif

#include <net/if.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#ifdef SUNOS4
#include <netinet/in_var.h>
#endif
#include <netinet/if_ether.h>

#include <errno.h>
#include <fcntl.h>
#include <kvm.h>
#include <nlist.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define printf Printf
#include <stdio.h>
#undef printf

#ifdef LBL
#include <lbl/os-proto.h>
#endif

#include "bootbootp.h"

/* Cast a struct sockaddr to a sockaddr_in */
#define SATOSIN(sa) ((struct sockaddr_in *)(sa))

/* Determine if "bits" is set in "flag" */
#define ALLSET(flag, bits) (((flag) & (bits)) == (bits))

static struct nlist nl[] = {
#define N_IFNET 0
	{ "_ifnet" },
	{ NULL }
};
static kvm_t *kd;

/* Forwards */
static void kread __P((u_long, char *, u_int, char *));

void
ethernet_macaddr(d, len, ea)
	register struct iodesc *d;
	int len;
	u_char *ea;
{
	register struct ifnet *ifp;
	register struct arpcom *ac;
	register u_char *ep;
	struct arpcom arpcom;
#ifdef SUNOS4
	register struct ifaddr *ifa;
	register struct in_ifaddr *in;
	union {
		struct ifaddr ifa;
		struct in_ifaddr in;
	} ifaddr;
#endif
	u_long addr;
	int num;


	if (len != 6)
		panic("ethernet_macaddr: bad buffer size %d", len);

	/* Open kernel memory for reading */
	kd = kvm_open(0, 0, 0, O_RDONLY, NULL);
	if (kd == 0)
		panic("ethernet_macaddr: kvm_open");

	/* Fetch namelist */
	if (kvm_nlist(kd, nl) != 0)
		panic("ethernet_macaddr: bad nlist");

	ac = &arpcom;
	ifp = &arpcom.ac_if;
	ep = arpcom.ac_enaddr.ether_addr_octet;
#ifdef SUNOS4
	ifa = &ifaddr.ifa;
	in = &ifaddr.in;
#endif
	num = 0;
	kread(nl[N_IFNET].n_value, (char *)&addr, sizeof(addr), "ifnet addr");
	for ( ; addr; addr = (u_long)ifp->if_next) {
		kread(addr, (char *)ac, sizeof(*ac), "ifnet");
		/* Only look at configured, broadcast interfaces */
		if (!ALLSET(ifp->if_flags, IFF_UP | IFF_BROADCAST))
			continue;
#ifdef SUNOS4
		/* This probably can't happen... */
		if (ifp->if_addrlist == 0)
			continue;
#endif
		bcopy(ep, ea, 6);
		return;
	}
	panic("ethernet_macaddr: can't find ethernet addr");
}

static void
kread(addr, buf, size, errmsg)
	u_long addr;
	char *buf;
	u_int size;
	char *errmsg;
{
	if (kvm_read(kd, addr, buf, size) != size)
		panic("kread: %s read", errmsg);
}
